@@ -35,9 +35,17 @@
 #include "gconf.h"
 #include "gconfd.h"
 #include "gconf-database.h"
+
+#ifdef HAVE_DBUS
+#include "gconf-database-dbus.h"
+#include "gconfd-dbus.h"
+#endif
+
+#ifdef HAVE_CORBA
 #include <orbit/orbit.h>
 
 #include "GConfX.h"
+#endif
 
 #include <sys/types.h>
 #include <stdio.h>
@@ -121,9 +129,9 @@ safe_g_hash_table_insert(GHashTable* ht, gpointer key, gpointer value)
  */
 
 static void     gconf_main            (void);
-static void     gconf_main_quit       (void);
 static gboolean gconf_main_is_running (void);
 
+#ifdef HAVE_CORBA
 static void logfile_save (void);
 static void logfile_read (void);
 static void log_client_add (const ConfigListener client);
@@ -135,17 +143,19 @@ static GSList *list_clients          (void);
 static void    log_clients_to_string (GString              *str);
 static void    drop_old_clients      (void);
 static guint   client_count          (void);
+#endif
 
 static void    enter_shutdown          (void);
 
 static void                 init_databases (void);
 static void                 shutdown_databases (void);
+#ifdef HAVE_DBUS
+static void                 reload_databases (void);
+#endif
 static void                 set_default_database (GConfDatabase* db);
 static void                 register_database (GConfDatabase* db);
 static void                 unregister_database (GConfDatabase* db);
 static GConfDatabase*       lookup_database (GSList *addresses);
-static GConfDatabase*       obtain_database (GSList *addresses,
-                                             GError **err);
 static void                 drop_old_databases (void);
 static gboolean             no_databases_in_use (void);
 
@@ -164,6 +174,7 @@ static gboolean in_shutdown = FALSE;
  */
 static gboolean need_db_reload = FALSE;
 
+#ifdef HAVE_CORBA
 /* 
  * CORBA goo
  */
@@ -256,7 +267,7 @@ gconfd_get_database(PortableServer_Servant servant,
     return CORBA_OBJECT_NIL;
   
   addresses = g_slist_append (NULL, (char *) address);
-  db = obtain_database (addresses, &error);
+  db = gconfd_obtain_database (addresses, &error);
   g_slist_free (addresses);
 
   if (db != NULL)
@@ -284,7 +295,7 @@ gconfd_get_database_for_addresses (PortableServer_Servant           servant,
   while (i < seq->_length)
     addresses = g_slist_append (addresses, seq->_buffer [i++]);
 
-  db = obtain_database (addresses, &error);
+  db = gconfd_obtain_database (addresses, &error);
 
   g_slist_free (addresses);
 
@@ -335,8 +346,9 @@ gconfd_shutdown(PortableServer_Servant servant, CORBA_Environment *ev)
   
   gconf_log(GCL_DEBUG, _("Shutdown request received"));
 
-  gconf_main_quit();
+  gconfd_main_quit();
 }
+#endif /* HAVE_CORBA */
 
 /*
  * Main code
@@ -344,8 +356,8 @@ gconfd_shutdown(PortableServer_Servant servant, CORBA_Environment *ev)
 
 /* This needs to be called before we register with OAF
  */
-static void
-gconf_server_load_sources(void)
+static GConfSources *
+gconf_server_get_default_sources(void)
 {
   GSList* addresses;
   GList* tmp;
@@ -397,8 +409,7 @@ gconf_server_load_sources(void)
       /* don't request error since there aren't any addresses */
       sources = gconf_sources_new_from_addresses(NULL, NULL);
 
-      /* Install the sources as the default database */
-      set_default_database (gconf_database_new(sources));
+      return sources;
     }
   else
     {
@@ -437,12 +448,21 @@ gconf_server_load_sources(void)
       if (!have_writable)
         gconf_log(GCL_WARNING, _("No writable configuration sources successfully resolved. May be unable to save some configuration changes"));
 
-        
-      /* Install the sources as the default database */
-      set_default_database (gconf_database_new(sources));
+      return sources;
     }
 }
 
+static void
+gconf_server_load_sources(void)
+{
+  GConfSources* sources;
+
+  sources = gconf_server_get_default_sources();
+
+  /* Install the sources as the default database */
+  set_default_database (gconf_database_new(sources));
+}
+
 /*
  * Signal handlers should not log debug messages as this code is non-reentrant.
  * Please avoid calling gconf_log in this function.
@@ -472,7 +492,7 @@ signal_handler (int signo)
     --in_fatal;
     
     if (gconf_main_is_running ())
-      gconf_main_quit ();
+      gconfd_main_quit ();
     
     break;
 
@@ -483,7 +503,7 @@ signal_handler (int signo)
     --in_fatal;
     
     if (gconf_main_is_running ())
-      gconf_main_quit ();
+      gconfd_main_quit ();
     break;
 
 #ifdef SIGHUP
@@ -512,12 +532,15 @@ signal_handler (int signo)
   }
 }
 
+#ifdef HAVE_CORBA
 PortableServer_POA
 gconf_get_poa (void)
 {
   return the_poa;
 }
+#endif
 
+#ifdef HAVE_CORBA
 static const char *
 get_introspection_xml (void)
 {
@@ -550,7 +573,7 @@ bus_message_handler (DBusConnection *connection,
                               DBUS_INTERFACE_LOCAL,
                               "Disconnected"))
     {
-      gconf_main_quit ();
+      gconfd_main_quit ();
       return DBUS_HANDLER_RESULT_HANDLED;
     }
   else if (dbus_message_is_method_call (message,
@@ -635,6 +658,7 @@ get_on_d_bus (void)
 
   return connection;
 }
+#endif
 
 #ifdef ENABLE_DEFAULTS_SERVICE
 /* listen on system bus for defaults changes */
@@ -667,7 +691,7 @@ system_bus_message_handler (DBusConnection *connection,
 
 	  gconf_log (GCL_DEBUG, "System defaults changed.  Notifying.");
 
-	  addresses.data = "xml:merged:/etc/gconf/gconf.xml.system";
+	  addresses.data = "xml:merged:" GCONF_ETCDIR "/gconf.xml.system";
 	  addresses.next = NULL;
 	  system_sources = gconf_sources_new_from_addresses (&addresses, NULL);
 
@@ -752,14 +776,17 @@ main(int argc, char** argv)
   sigset_t empty_mask;
   sigset_t full_mask;
 #endif
+
+#ifdef HAVE_CORBA
   CORBA_Environment ev;
   CORBA_ORB orb;
   gchar* ior;
-  int exit_code = 0;
-  GError *err;
-  int dev_null_fd;
-  int write_byte_fd;
   DBusConnection *connection;
+#endif
+
+  int dev_null_fd;
+  int exit_code = 0;
+  int write_byte_fd;
 
   _gconf_init_i18n ();
   setlocale (LC_ALL, "");
@@ -871,10 +898,18 @@ main(int argc, char** argv)
 #endif
 #endif
 
+#ifdef HAVE_CORBA
   CORBA_exception_init(&ev);
+#endif
 
   init_databases ();
 
+#ifdef HAVE_DBUS
+  if (!gconfd_dbus_init ())
+    return 1;
+#endif
+
+#ifdef HAVE_CORBA
   orb = gconf_orb_get ();
   
   POA_ConfigServer2__init (&poa_server_servant, &ev);
@@ -907,7 +942,13 @@ main(int argc, char** argv)
        */
       gconf_server_load_sources ();
     }
-  
+#endif
+
+#ifdef HAVE_DBUS
+  gconf_server_load_sources ();
+#endif
+
+#ifdef HAVE_CORBA
   /* notify caller that we're done either getting the lock
    * or not getting it
    */
@@ -932,7 +973,8 @@ main(int argc, char** argv)
 
   /* Read saved log file, if any */
   logfile_read ();
- 
+#endif
+
 #ifdef ENABLE_DEFAULTS_SERVICE 
   get_on_system_bus ();
 #endif
@@ -947,17 +989,22 @@ main(int argc, char** argv)
    */
   enter_shutdown ();
 
+#ifdef HAVE_CORBA
   /* Save current state in logfile (may compress the logfile a good
    * bit)
    */
   logfile_save ();
+#endif
   
   shutdown_databases ();
 
   gconfd_locale_cache_drop ();
 
+#ifdef HAVE_CORBA
   if (daemon_lock)
     {
+      GError *err;
+
       err = NULL;
       gconf_release_lock (daemon_lock, &err);
       if (err != NULL)
@@ -969,6 +1016,7 @@ main(int argc, char** argv)
     }
 
   daemon_lock = NULL;
+#endif
   
   gconf_log (GCL_DEBUG, _("Exiting"));
 
@@ -991,28 +1039,40 @@ periodic_cleanup_timeout(gpointer data)
       gconf_log (GCL_INFO, _("SIGHUP received, reloading all databases"));
 
       need_db_reload = FALSE;
+#ifdef HAVE_CORBA
       logfile_save ();
       shutdown_databases ();
       init_databases ();
       gconf_server_load_sources ();
       logfile_read ();
+#endif
+#ifdef HAVE_DBUS
+      reload_databases ();
+#endif
     }
   
   gconf_log (GCL_DEBUG, "Performing periodic cleanup, expiring cache cruft");
   
+#ifdef HAVE_CORBA
   drop_old_clients ();
+#endif
   drop_old_databases ();
 
+#ifdef HAVE_DBUS
+  if (no_databases_in_use () && gconfd_dbus_client_count () == 0)
+#else
   if (no_databases_in_use () && client_count () == 0)
+#endif
     {
       gconf_log (GCL_INFO, _("GConf server is not in use, shutting down."));
-      gconf_main_quit ();
+      gconfd_main_quit ();
       return FALSE;
     }
   
   /* expire old locale cache entries */
   gconfd_locale_cache_expire ();
 
+#ifdef HAVE_CORBA
   if (!need_log_cleanup)
     {
       gconf_log (GCL_DEBUG, "No log file saving needed in periodic cleanup handler");
@@ -1021,6 +1081,7 @@ periodic_cleanup_timeout(gpointer data)
   
   /* Compress the running state file */
   logfile_save ();
+#endif
 
   need_log_cleanup = FALSE;
   
@@ -1067,8 +1128,8 @@ gconf_main(void)
   g_main_loop_unref (loop);
 }
 
-static void 
-gconf_main_quit(void)
+void
+gconfd_main_quit(void)
 {
   g_return_if_fail(main_loops != NULL);
 
@@ -1157,9 +1218,9 @@ lookup_database (GSList *addresses)
   return retval;
 }
 
-static GConfDatabase*
-obtain_database (GSList  *addresses,
-                 GError **err)
+GConfDatabase *
+gconfd_obtain_database (GSList  *addresses,
+                        GError **err)
 {
   GConfSources* sources;
   GError* error = NULL;
@@ -1219,6 +1280,10 @@ drop_old_databases(void)
       
       if (db->listeners &&                             /* not already hibernating */
           gconf_listeners_count(db->listeners) == 0 && /* Can hibernate */
+#ifdef HAVE_DBUS
+          db->listening_clients &&
+          g_hash_table_size (db->listening_clients) == 0 &&
+#endif
           (now - db->last_access) > (60*20))           /* 20 minutes without access */
         {
           dead = g_list_prepend (dead, db);
@@ -1270,6 +1335,56 @@ shutdown_databases (void)
   default_db = NULL;
 }
 
+#ifdef HAVE_DBUS
+static void
+reload_databases (void)
+{
+  GConfSources* sources;
+  GList *tmp_list;
+
+  sources = gconf_server_get_default_sources ();
+  gconf_database_set_sources (default_db, sources);
+
+  tmp_list = db_list;
+  while (tmp_list)
+    {
+      GConfDatabase* db = tmp_list->data;
+      GList *l;
+      GConfSource *source;
+      GSList *addresses = NULL;
+      GError *error = NULL;
+
+      if (db == default_db)
+	{
+	  tmp_list = g_list_next (tmp_list);
+	  continue;
+	}
+
+      for (l = db->sources->sources; l != NULL; l = l->next)
+        {
+          source = l->data;
+          addresses = g_slist_prepend (addresses, source->address);
+        }
+
+      addresses = g_slist_reverse (addresses);
+      sources = gconf_sources_new_from_addresses (addresses, &error);
+
+      if (error == NULL)
+        {
+          gconf_database_set_sources (db, sources);
+        }
+      else
+        {
+          /* if we got an error, keep our old sources -- that's better than
+           * nothing */
+          g_error_free (error);
+        }
+
+      tmp_list = g_list_next (tmp_list);
+    }
+}
+#endif
+
 static gboolean
 no_databases_in_use (void)
 {
@@ -1314,7 +1429,9 @@ gconfd_notify_other_listeners (GConfDatabase *modified_db,
 	      if (gconf_sources_is_affected (db->sources, modified_source, key))
 		{
 		  GConfValue  *value;
+#ifdef HAVE_CORBA
 		  ConfigValue *cvalue;
+#endif
 		  GError      *error;
 		  gboolean     is_default;
 		  gboolean     is_writable;
@@ -1337,6 +1454,7 @@ gconfd_notify_other_listeners (GConfDatabase *modified_db,
 		      return;
 		    }
 
+#if HAVE_CORBA
 		  if (value != NULL)
 		    {
 		      cvalue = gconf_corba_value_from_gconf_value (value);
@@ -1355,6 +1473,16 @@ gconfd_notify_other_listeners (GConfDatabase *modified_db,
 						   is_writable,
 						   FALSE);
 		  CORBA_free (cvalue);
+#endif
+#ifdef HAVE_DBUS
+		  gconf_database_dbus_notify_listeners (db,
+							NULL,
+							key,
+							value,
+							is_default,
+							is_writable,
+							FALSE);
+#endif
 		}
 
 	      tmp2 = tmp2->next;
@@ -1392,6 +1520,7 @@ enter_shutdown(void)
 }
 
 
+#ifdef HAVE_CORBA
 /* Exceptions */
 
 gboolean
@@ -2212,7 +2341,7 @@ listener_logentry_restore_and_destroy_foreach (gpointer key,
 
       addresses = gconf_persistent_name_get_address_list (lle->address);
 
-      db = obtain_database (addresses, NULL);
+      db = gconfd_obtain_database (addresses, NULL);
 
       gconf_address_list_free (addresses);
     }
@@ -2709,4 +2838,10 @@ client_count (void)
   else
     return g_hash_table_size (client_table);
 }
+#endif
 
+gboolean
+gconfd_in_shutdown (void)
+{
+  return in_shutdown;
+}
